@@ -12,15 +12,16 @@ import numpy as np
 
 import pythesint as pti
 
-from nansat.tools import gdal, ogr, WrongMapperError
+from nansat.tools import gdal, ogr
+from nansat.exceptions import WrongMapperError
 from nansat.vrt import VRT
-from hdf4_mapper import HDF4Mapper
+from nansat.mappers.hdf4_mapper import HDF4Mapper
 
 
 class Mapper(HDF4Mapper):
     ''' VRT with mapping of WKV for MODIS Level 1 (QKM, HKM, 1KM) '''
 
-    def __init__(self, fileName, gdalDataset, gdalMetadata, GCP_COUNT=30, **kwargs):
+    def __init__(self, filename, gdalDataset, gdalMetadata, GCP_COUNT=30, **kwargs):
         ''' Create MODIS_L1 VRT '''
 
         #list of available modis names:resolutions
@@ -42,7 +43,7 @@ class Mapper(HDF4Mapper):
             raise WrongMapperError
 
         # create empty VRT dataset with geolocation only
-        VRT.__init__(self, gdalSubDataset)
+        self._init_from_gdal_dataset(gdalSubDataset)
 
         subDsString = 'HDF4_EOS:EOS_SWATH:"%s":MODIS_SWATH_Type_L1B:%s'
 
@@ -50,12 +51,12 @@ class Mapper(HDF4Mapper):
         metaDict250SF = ['EV_250_RefSB']
 
         metaDict250 = [{'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_250_RefSB'),
+                                (filename, 'EV_250_RefSB'),
                                 'SourceBand': 1},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '645'}},
                        {'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_250_RefSB'),
+                                (filename, 'EV_250_RefSB'),
                                 'SourceBand': 2},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '858'}}]
@@ -63,38 +64,38 @@ class Mapper(HDF4Mapper):
         metaDict500SF = ['EV_250_Aggr500_RefSB', 'EV_500_RefSB']
 
         metaDict500 = [{'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_250_Aggr500_RefSB'),
+                                (filename, 'EV_250_Aggr500_RefSB'),
                                 'SourceBand': 1},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '645'}},
                        {'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_250_Aggr500_RefSB'),
+                                (filename, 'EV_250_Aggr500_RefSB'),
                                 'SourceBand': 2},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '858'}},
 
                        {'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_500_RefSB'),
+                                (filename, 'EV_500_RefSB'),
                                 'SourceBand': 1},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '469'}},
                        {'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_500_RefSB'),
+                                (filename, 'EV_500_RefSB'),
                                 'SourceBand': 2},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '555'}},
                        {'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_500_RefSB'),
+                                (filename, 'EV_500_RefSB'),
                                 'SourceBand': 3},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '1240'}},
                        {'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_500_RefSB'),
+                                (filename, 'EV_500_RefSB'),
                                 'SourceBand': 4},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '1640'}},
                        {'src': {'SourceFilename': subDsString %
-                                (fileName, 'EV_500_RefSB'),
+                                (filename, 'EV_500_RefSB'),
                                 'SourceBand': 5},
                         'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                 'wavelength': '2130'}}]
@@ -103,195 +104,195 @@ class Mapper(HDF4Mapper):
                           'EV_1KM_RefSB', 'EV_1KM_Emissive']
 
         metaDict1000 = [{'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_250_Aggr1km_RefSB'),
+                                 (filename, 'EV_250_Aggr1km_RefSB'),
                                  'SourceBand': 1},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '645'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_250_Aggr1km_RefSB'),
+                                 (filename, 'EV_250_Aggr1km_RefSB'),
                                  'SourceBand': 2},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '858'}},
 
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_500_Aggr1km_RefSB'),
+                                 (filename, 'EV_500_Aggr1km_RefSB'),
                                  'SourceBand': 1},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '469'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_500_Aggr1km_RefSB'),
+                                 (filename, 'EV_500_Aggr1km_RefSB'),
                                  'SourceBand': 2},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '555'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_500_Aggr1km_RefSB'),
+                                 (filename, 'EV_500_Aggr1km_RefSB'),
                                  'SourceBand': 3},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '1240'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_500_Aggr1km_RefSB'),
+                                 (filename, 'EV_500_Aggr1km_RefSB'),
                                  'SourceBand': 4},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '1640'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_500_Aggr1km_RefSB'),
+                                 (filename, 'EV_500_Aggr1km_RefSB'),
                                  'SourceBand': 5},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '2130'}},
 
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 1},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '412'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 2},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '443'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 3},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '488'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 4},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '531'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 5},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '551'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 6},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '667'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 7},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '667'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 8},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '678'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 9},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '678'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 10},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '748'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 11},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '869'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 12},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '905'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 13},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '936'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 14},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '940'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_RefSB'),
+                                 (filename, 'EV_1KM_RefSB'),
                                  'SourceBand': 15},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '1375'}},
 
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 1},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '3750'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 2},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '3959'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 3},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '3959'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 4},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '4050'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 5},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '4465'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 6},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '4515'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 7},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '6715'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 8},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '7325'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 9},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '8550'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 10},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '9730'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 11},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '11030'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 12},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '12020'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 13},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '13335'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 14},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '13635'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 15},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '13935'}},
                         {'src': {'SourceFilename': subDsString %
-                                 (fileName, 'EV_1KM_Emissive'),
+                                 (filename, 'EV_1KM_Emissive'),
                                  'SourceBand': 16},
                          'dst': {'wkv': 'toa_outgoing_spectral_radiance',
                                  'wavelength': '14235'}}]
@@ -311,14 +312,14 @@ class Mapper(HDF4Mapper):
         rScales = {}
         rOffsets = {}
         for sf in metaDictSF:
-            dsName = subDsString % (fileName, sf)
+            dsName = subDsString % (filename, sf)
             ds = gdal.Open(dsName)
-            rScales[dsName] = map(float,
+            rScales[dsName] = list(map(float,
                                   ds.GetMetadataItem('radiance_scales').
-                                  split(','))
-            rOffsets[dsName] = map(float,
+                                  split(',')))
+            rOffsets[dsName] = list(map(float,
                                    ds.GetMetadataItem('radiance_offsets').
-                                   split(','))
+                                   split(',')))
             self.logger.debug('radiance_scales: %s' % str(rScales))
 
         # add 'band_name' to 'parameters'
@@ -334,11 +335,11 @@ class Mapper(HDF4Mapper):
             bandDict['src']['ScaleOffset'] = offset
 
         # add bands with metadata and corresponding values to the empty VRT
-        self._create_bands(metaDict)
+        self.create_bands(metaDict)
 
         productDate = gdalMetadata["RANGEBEGINNINGDATE"]
         productTime = gdalMetadata["RANGEBEGINNINGTIME"]
-        self.remove_geolocationArray()
+        self._remove_geolocation()
 
         # set required metadata
         self.dataset.SetMetadataItem('time_coverage_start',
